@@ -21,7 +21,7 @@ class Bug(object):
     def search(cls, bz, args):
         pass
 
-    def __init__(self, bz, bugno_or_data):
+    def __init__(self, bz, bugno_or_data=None):
         """Create a bug object.
 
         bz: a bzlib.Bugzilla object
@@ -36,10 +36,10 @@ class Bug(object):
         self.bugno = None
         self.data = None
         self.comments = None
-        if isinstance(bugno_or_data, int):
-            self.bugno = bugno_or_data
-        elif bugno_or_data:
-            self.data = bugno_or_data
+        try:
+            self.bugno = int(bugno_or_data)
+        except TypeError:
+            self.data = bugno_or_data or {}
 
     def rpc(self, *args, **kwargs):
         """Does an RPC on the Bugzilla server.
@@ -78,11 +78,14 @@ class Bug(object):
           - status
           - priority
           - cc
+
+        Return the new bug ID.
         """
         if self.bugno or 'id' in self.data:
             raise Exception("bugno is known; not creating bug.")
         result = self.rpc('create', **self.data)
         self.bugno = result['id']
+        return self.bugno
 
     def get_comments(self):
         self.read_comments()
@@ -92,6 +95,21 @@ class Bug(object):
         self.rpc('add_comment', id=self.bugno, comment=comment)
         self.comments = None  # comments are stale
 
+    def is_open(self):
+        """Return True if the bug is open, otherwise False."""
+        self.read()
+        return self.data['is_open']
+
+    def set_dupe_of(self, bug, comment=None):
+        """Set this bug a duplicate of the given bug."""
+        kwargs = {'dupe_of': bug}
+        if comment:
+            kwargs['comment'] = {'body': comment}
+        self.rpc('update', ids=[self.bugno], **kwargs)
+        self.data = None  # data is stale
+        if comment:
+            self.comments = None  # comments are stale
+
     def set_status(self, status, resolution='', comment=None):
         """Set the status of this bug.
 
