@@ -20,6 +20,7 @@ import argparse
 import itertools
 import textwrap
 
+from . import bug
 from . import bugzilla
 from . import config
 from . import editor
@@ -75,26 +76,6 @@ def with_optional_message(cls):
     return cls
 
 
-def with_status(cls):
-    cls.args = cls.args + [
-        lambda x: x.add_argument('--status',
-            help='Specify a resolution (case-insensitive).'),
-        lambda x: x.add_argument('--choose-status', action='store_true',
-            help='Choose status from a list.')
-    ]
-    return cls
-
-
-def with_resolution(cls):
-    cls.args = cls.args + [
-        lambda x: x.add_argument('--resolution',
-            help='Specify a resolution (case-insensitive).'),
-        lambda x: x.add_argument('--choose-resolution', action='store_true',
-            help='Choose resolution from a list.')
-    ]
-    return cls
-
-
 def with_limit(things='items', default=None):
     def decorator(cls):
         cls.args = cls.args + [
@@ -415,17 +396,100 @@ class List(BugzillaCommand):
             )
 
 
-#class New(BugzillaCommand):
-#    """File a new bug."""
-#    pass
+class New(BugzillaCommand):
+    """File a new bug."""
+    def __call__(self):
+        # create new Bug
+        b = bug.Bug(self.bz)
+
+        # get mandatory fields
+        fields = self.bz.get_fields()
+        mandatory_fields = filter(lambda x: x['is_mandatory'], fields)
+
+        # first choose the product
+        b.data['product'] = self._ui.choose(
+            'Choose a product',
+            map(lambda x: x['name'], self.bz.get_products())
+        )
+
+        # fill out other mandatory fields
+        for field in mandatory_fields:
+            if field['name'] in b.data:
+                continue  # field is already defined
+            if 'values' in field:
+                values = self.bz.get_field_values(
+                    field['name'],
+                    visible_for=b.data
+                )
+                # TODO handle select-multiple fields
+                b.data[field['name']] = self._ui.choose(
+                    'Choose the {}'.format(field['display_name']),
+                    map(lambda x: x['name'], values)
+                )
+            else:
+                # TODO take field types into account
+                b.data[field['name']] = self._ui.text(
+                    'Enter the {}'.format(field['display_name'])
+                )
+
+        # fill out a comment ("Description") if not already defined
+        if 'comment' not in b.data:
+            b.data['comment'] = editor.input(
+                'Enter a description of the problem.'
+            )
+
+        # list of create fields ripped from Bugzilla documentation
+        # (this info is not introspectable as of Bugzilla 4.0)
+        create_fields = [
+            'product', 'component', 'summary', 'version', 'comment',
+            'op_sys', 'platform', 'priority', 'severity', 'alias',
+            'assigned_to', 'cc', 'comment_is_private', 'groups',
+            'qa_contact', 'status', 'target_milestone',
+        ]
+
+        # let user choose which of these other fields to define
+        optional_fields = [ \
+            x['display_name'] for x in fields \
+            if x['name'] in create_fields \
+                and x['name'] not in b.data.viewkeys()
+        ]
+
+        # prompt user for the fields they wish to define
+        user_fields = self._ui.chooseN(
+            'Set values for other fields?',
+            optional_fields,
+            default=[]
+        )
+
+        # iterate over those fields, prompting user for values
+        for field in [x for x in fields if x['display_name'] in user_fields]:
+            if field['name'] in b.data:
+                continue  # field is already defined
+            if 'values' in field:
+                values = self.bz.get_field_values(
+                    field['name'],
+                    visible_for=b.data
+                )
+                # TODO handle select-multiple fields
+                b.data[field['name']] = self._ui.choose(
+                    'Choose the {}'.format(field['display_name']),
+                    map(lambda x: x['name'], values)
+                )
+            else:
+                # TODO take field types into account
+                b.data[field['name']] = self._ui.text(
+                    'Enter the {}'.format(field['display_name'])
+                )
+
+        # create the bug
+        id = b.create()
+        self._ui.show('Created Bug {}'.format(id))
 
 
 class Products(BugzillaCommand):
     """List the products of a Bugzilla instance."""
     def __call__(self):
-        args = self._args
-        ids = self.bz.rpc('Product', 'get_accessible_products')['ids']
-        products = self.bz.rpc('Product', 'get', ids=ids)['products']
+        products = self.bz.get_products()
         width = max(map(lambda x: len(x['name']), products)) + 1
         for product in products:
             print '{:{}} {}'.format(
@@ -434,22 +498,54 @@ class Products(BugzillaCommand):
 
 
 @with_bugs
-@with_status
-@with_resolution
 @with_optional_message
 class Status(BugzillaCommand):
-    """Set the status of the given bugs."""
+    """Set the status of the given bugs.
+
+    Description
+    -----------
+
+    The ``status`` command is used to update the status and resolution of
+    bugs.  The status is always required unless ``-dupe-of`` is used (see
+    below).  It can be given as the argument to ``--status``, otherwise the
+    user will be prompted to choose the status from a list.
+
+    If the status is changing from one considered "open" to one not
+    considered "open", a resolution is required.  It can be given using
+    ``--resolution``, otherwise the user will be prompted to choose the
+    resolution from a list.
+
+    Marking bugs as duplicates
+    --------------------------
+
+    To set a bug as a duplicate, simply use ``--dupe-of <BUG>``.  ``--status``
+    and ``--resolution`` will be ignored.  Bugzilla will automatically set the
+    status and resolution fields to appropriate values for duplicate bugs.
+    """
+
+    args = [
+        lambda x: x.add_argument('--status',
+            help='Specify a resolution (case-insensitive).'),
+        lambda x: x.add_argument('--resolution',
+            help='Specify a resolution (case-insensitive).'),
+        lambda x: x.add_argument('--dupe-of', type=int, metavar='BUG',
+            help='The bug of which the given bugs are duplicates.'),
+    ]
+
     def __call__(self):
         args = self._args
         message = editor.input('Enter your comment.') if args.message is True \
             else args.message
 
+        if args.dupe_of:
+            # This is all we need; --status and --resolution are ignored
+            return map(
+                lambda x: self.bz.bug(x).set_dupe_of(args.dupe_of, message),
+                args.bugs
+            )
+
         # get the values of the 'bug_status' field
-        values = filter(
-            lambda x: x['name'] == 'bug_status',
-            self.bz.get_fields()
-        )[0]['values']
-        values = sorted(values, None, lambda x: int(x['sortkey']))
+        values = self.bz.get_field_values('bug_status')
 
         if args.status:
             status = args.status.upper()
@@ -457,7 +553,7 @@ class Status(BugzillaCommand):
             # choose status
             status = self._ui.choose(
                 'Choose a status',
-                filter(None, map(lambda x: x['name'], values))
+                map(lambda x: x['name'], values)
             )
 
         # check if the new status is "open"
@@ -468,20 +564,23 @@ class Status(BugzillaCommand):
             # no value matching the chosen status
             raise UserWarning("Invalid status:", status)
 
+        # instantiate bugs
+        bugs = map(self.bz.bug, args.bugs)
+
         resolution = None
         if not is_open:
-            # resolution required
+            # The new status accepts a resolution.
             if args.resolution:
+                # A resolution was supplied.
                 resolution = args.resolution.upper()
-            else:
-                # choose resolution
-                values = filter(
-                    lambda x: x['name'] == 'resolution',
-                    self.bz.get_fields()
-                )[0]['values']
-                values = sorted(values, None, lambda x: int(x['sortkey']))
-                values = filter(None, map(lambda x: x['name'], values))
-                resolution = self._ui.choose('Choose a resolution', values)
+            elif any(map(lambda x: x.is_open(), bugs)):
+                # A resolution was not supplied, but one is required since
+                # at least one of the bugs is currently open.  Choose one.
+                values = self.bz.get_field_values('resolution')
+                resolution = self._ui.choose(
+                    'Choose a resolution',
+                    map(lambda x: x['name'], values)
+                )
 
         return map(
             lambda x: self.bz.bug(x).set_status(
